@@ -1,13 +1,12 @@
 #pragma once
 
-#include <ros/ros.h>
-#include <vision_msgs/Detection2DArray.h>
-
-#include <boost/make_shared.hpp>
-#include <boost/shared_ptr.hpp>
-#include <depthai/depthai.hpp>
-#include <depthai_bridge/depthaiUtility.hpp>
 #include <deque>
+#include <memory>
+#include <string>
+
+#include "depthai/pipeline/datatype/ImgDetections.hpp"
+#include "ros/time.h"
+#include "vision_msgs/Detection2DArray.h"
 
 namespace dai {
 
@@ -18,7 +17,24 @@ using Detection2DArrayPtr = VisionMsgs::Detection2DArray::Ptr;
 class ImgDetectionConverter {
    public:
     // DetectionConverter() = default;
-    ImgDetectionConverter(std::string frameName, int width, int height, bool normalized = false);
+    ImgDetectionConverter(std::string frameName, int width, int height, bool normalized = false, bool getBaseDeviceTimestamp = false);
+
+    /**
+     * @brief Handles cases in which the ROS time shifts forward or backward
+     *  Should be called at regular intervals or on-change of ROS time, depending
+     *  on monitoring.
+     *
+     */
+    void updateRosBaseTime();
+
+    /**
+     * @brief Commands the converter to automatically update the ROS base time on message conversion based on variable
+     *
+     * @param update: bool whether to automatically update the ROS base time on message conversion
+     */
+    void setUpdateRosBaseTimeOnToRosMsg(bool update = true) {
+        _updateRosBaseTimeOnToRosMsg = update;
+    }
 
     void toRosMsg(std::shared_ptr<dai::ImgDetections> inNetData, std::deque<VisionMsgs::Detection2DArray>& opDetectionMsgs);
 
@@ -30,6 +46,11 @@ class ImgDetectionConverter {
     bool _normalized;
     std::chrono::time_point<std::chrono::steady_clock> _steadyBaseTime;
     ::ros::Time _rosBaseTime;
+    bool _getBaseDeviceTimestamp;
+    // For handling ROS time shifts and debugging
+    int64_t _totalNsChange{0};
+    // Whether to update the ROS base time on each message conversion
+    bool _updateRosBaseTimeOnToRosMsg{false};
 };
 
 /** TODO(sachin): Do we need to have ros msg -> dai bounding box ?
